#include "bspline.h"

BSpline::BSpline(int k, double step, int np1)
{
    // Initialisation des valeurs
    // Ordre k = 2
    _k = k;
    _step = step;
    _np1 = np1;
    // Vecteur Nodal Uniforme de np1 valeurs
    for(int i = 0; i < k+np1; ++i)
        _vecteurNodal.push_back(i);
    // Points de contrôle
    _pointsDeControle.push_back(glm::vec3(-2, 0, -5));
    _pointsDeControle.push_back(glm::vec3(-1, 0, -5));
    _pointsDeControle.push_back(glm::vec3(-1, 1, -5));
    _pointsDeControle.push_back(glm::vec3( 1, 1, -5));
    _pointsDeControle.push_back(glm::vec3( 1, 0, -5));
    _pointsDeControle.push_back(glm::vec3( 2, 0, -5));

    computeSpline();
}

void BSpline::computeSpline()
{
    // Init
    int dec = 0, i = _k;
    double u = 0;

    std::vector<glm::vec3> vecteursPointsControle (_np1);

    for(u = _k - 1; u < _np1; u += _step){
        // Réinitialiser dec et i ?
        while(u > _vecteurNodal[i]){
            ++dec;
            ++i;
        }

        // int i à virer ?
        for(int i = 0; i < _k; i++){
            vecteursPointsControle[i] = _pointsDeControle[dec + i];
        }

        glm::vec3 newVertex = floraison(u, dec, _k, vecteursPointsControle);
        //std::cout << newVertex.x << "  " << newVertex.y << "  " << newVertex.z << std::endl;
        _vertices.push_back(newVertex);
    }
}

glm::vec3 BSpline::floraison(float u, int dec, int k, std::vector<glm::vec3> vecteursPointsControle)
{
    if(k == 0)
        return vecteursPointsControle.front();
    for(int i = 0; i < k - 1; ++i)
        vecteursPointsControle[i] =
                (((_vecteurNodal[dec + k + i] - u) / (_vecteurNodal[dec + k + i] - _vecteurNodal[dec + 1 + i])) * vecteursPointsControle[i]) *
                (((u - _vecteurNodal[dec + 1 + i]) / (_vecteurNodal[dec + k + i] - _vecteurNodal[dec + 1 + i])) * vecteursPointsControle[i+1]);
    floraison(u, dec+1, k-1, vecteursPointsControle);
}

std::vector<GLfloat> const BSpline::getVertices()
{
    std::vector<GLfloat> vertices;
    // Ajout des points de contrôle
    for(glm::vec3 point: _pointsDeControle)
        vertices.insert(vertices.end(), {point.x, point.y, point.z});

    // Ajout des points issus de la floraison
    for(glm::vec3 point: _vertices)
        vertices.insert(vertices.end(), {point.x, point.y, point.z});

    return vertices;
}

std::vector<GLfloat> const BSpline::getNormals()
{
    std::vector<GLfloat> normals;
    for(glm::vec3 point: _pointsDeControle)
        normals.insert(normals.end(), {point.x, point.y, point.z});
    // Ajout des normales des points issus de la floraison
    for(glm::vec3 point: _vertices)
        normals.insert(normals.end(), {point.x, point.y, point.z});

    return normals;
}

std::vector<GLuint> const BSpline::getIndices()
{
    std::vector<GLuint> indices;
    for(uint i = 0; i < _pointsDeControle.size() - 1; ++i)
        indices.insert(indices.end(), {i, i, i+1});
    // Ajout des indices pour les points issus de la floraison
    for(uint i = 0; i < _vertices.size() - 1; ++i)
        indices.insert(indices.end(), {i, i, i+1});

    return indices;
}
